/*
 * arch/arm/mach-prima2/include/mach/io.h
 *
 * Copyright (c) 2011 Cambridge Silicon Radio Limited, a CSR plc group company.
 *
 * Licensed under GPLv2 or later.
 */

#ifndef __MACH_PRIMA2_IO_H
#define __MACH_PRIMA2_IO_H

#define IO_SPACE_LIMIT ((resource_size_t)0)

#define __mem_pci(a)            (a)

#endif
