/* linux/arch/arm/mach-s5p64x0/include/mach/spi-clocks.h
 *
 * Copyright (c) 2010 Samsung Electronics Co., Ltd.
 *		http://www.samsung.com
 *
 * Copyright (C) 2010 Samsung Electronics Co. Ltd.
 *	Jaswinder Singh <jassi.brar@samsung.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
*/

#ifndef __ASM_ARCH_SPI_CLKS_H
#define __ASM_ARCH_SPI_CLKS_H __FILE__

#define S5P64X0_SPI_SRCCLK_PCLK		0
#define S5P64X0_SPI_SRCCLK_SCLK		1

#endif /* __ASM_ARCH_SPI_CLKS_H */
